#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "heap4xi.py">
#<abstract4xi abstract = "provides class Heap4Xi"/>
#<refer4xi reference = "heap.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Heap4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass
try:                 # most-common ones
    import tempfile
    import itertools         
except:
    pass
try:                 # rare ones
    import bisect
    import mmap        
except:
    pass 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    print("heap4xi.py: *** failed in import _multiprocessing ***")
try:                             # Source ones
    from multiprocessing import heap
except:
    print("heap4xi.py: *** failed in from multiprocessing import heap ***") 
try:                             # Source ones
    from multiprocessing.heap import Heap
except:
    print("heap4xi.py: *** failed in from multiprocessing.heap import Heap ***")   
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from constant_heap4xi import ConstantHeap4Xi
from examine4xi import examine4xi
from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>


# =================================================================================================
#<class4xi class = "Heap4Xi" base = "object" child = "None" synopsis = "allows allocation of chunks of memory from arenas" usage = "
#    heap.py
#        BufferWrapper
#            _heap = Heap()
#
#    syncmanager4xi.py:
#        cheap2xi = ConstantHeap4Xi()
#
#    bufferwrapper4xi.py
#        BufferWrapper4Xi
#            _heap = Heap4Xi()
#            __init__(self, size):
#                block2xi = BufferWrapper4Xi._heap.malloc(size)
#    test_finalize4xi
#        f2xi = Finalize4Xi(b2xi, BufferWrapper4Xi._heap.free, args=(block2xi,))
#
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
#>
# =================================================================================================
class Heap4Xi(object):
    '''
    Heap4Xi class: allows allocation of chunks of memory from arenas
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('malloc','free','sense', 'reason', 'ace')    
    #"
    #/>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    a2xi = ConstantHeap4Xi()
    _alignment = a2xi.get_alignment()
    del a2xi
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init" usage = "initializes Heap4Xi with (self, size=mmap.PAGESIZE)">                           
    # ---------------------------------------------------------------------------------------------
    def __init__(self, size=mmap.PAGESIZE):
        '''
        Heap4Xi method __init__ initializes with (self, size=mmap.PAGESIZE) 
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/12) 
        # -----------------------------------------------------------------------------------------
        class2print = "Heap4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/12a) started" + "; init-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #heap4xi.Heap4Xi object at 0x02A49ED0
        
        message2print = "{(Step 1/12b) passed" + "; size-passed-to-init-of-Heap4Xi: " + str(size) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        #4096
        
        message2print = "{(Step 1/12c) borning" + "; heap2xi-of-Heap4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #heap4xi.Heap4Xi object at 0x02B6BB30
        
        message2print = "{(Step 1/12d) inheriting" + "; object-inheriting-by-heap2xi-of-Heap4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #class 'object'                
        
        message2print = "{(Step 1/12e) sharing" + "; alignment-sharing-by-heap2xi-of-Heap4Xi: " + str(Heap4Xi._alignment)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
                        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-init-performing-by-init-of-Heap4Xi: " + str(Heap.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            Heap.__init__(self, size)
            
            message2print = "{(StepOriginal) performed" + "; Heap-init-performed-by-init-of-Heap4Xi: " + str(Heap.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                        
        else:            
            # -----------------------------------------------------------------------------------------
            # (Step 2/12) 
            # -----------------------------------------------------------------------------------------             
            self._lastpid = os.getpid()
            
            message2print = "{(Step 2/12) set" + "; lastpid-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._lastpid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #3020
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/12) 
            #-----------------------------------------------------------------------------------------          
            self._lock = threading.Lock()
    
            message2print = "{(Step 3/12) set" + "; lock-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._lock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #_thread.lock object at 0x02996EE0
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/12) 
            # -----------------------------------------------------------------------------------------          
            self._size = size
            
            message2print = "{(Step 4/12) set" + "; size-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._size)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #4096
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/12) 
            # -----------------------------------------------------------------------------------------          
            self._lengths = []
            
            message2print = "{(Step 5/12) set" + "; lengths-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._lengths)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #[]
            
            # -----------------------------------------------------------------------------------------
            # (Step 6/12) 
            # -----------------------------------------------------------------------------------------          
            self._len_to_seq = {}
            
            message2print = "{(Step 6/12) set" + "; len-of-heap2xi-to-seq-set-by-init-of-Heap4Xi to: " + str(self._len_to_seq)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #{}
            
            # -----------------------------------------------------------------------------------------
            # (Step 7/12) 
            # -----------------------------------------------------------------------------------------          
            self._allocated_blocks = set()
            
            message2print = "{(Step 7/12) set" + "; allocated-blocks-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._allocated_blocks)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #set()
    
            # -----------------------------------------------------------------------------------------
            # (Step 8/12) 
            # -----------------------------------------------------------------------------------------          
            self._start_to_block = {}
            
            message2print = "{(Step 8/12) set" + "; start-to-block-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._start_to_block)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            #{}
                 
            # -----------------------------------------------------------------------------------------
            # (Step 9/12) 
            # -----------------------------------------------------------------------------------------          
            self._stop_to_block = {}
            
            message2print = "{(Step 9/12) set" + "; stop-to-block-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._stop_to_block)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
            #{}
            
            # -----------------------------------------------------------------------------------------
            # (Step 10/12) 
            # -----------------------------------------------------------------------------------------          
            self._arenas = []
            
            message2print = "{(Step 10/12) set" + "; arenas-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._arenas)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #[]
    
            # -----------------------------------------------------------------------------------------
            # (Step 11/12) - LIST OF PENDING BLOCKS TO FREE - SEE FREE() COMMENT BELOW
            # -----------------------------------------------------------------------------------------          
            self._pending_free_blocks = []
            
            message2print = "{ (Step 11/12) set" + "; pending-free-blocks-of-heap2xi-set-by-init-of-Heap4Xi to: " + str(self._pending_free_blocks)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #
                    
            # -----------------------------------------------------------------------------------------
            # (Step 12/12) 
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 12/12a) activated" + "; heap2xi-of-Heap4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #heap4xi.Heap4Xi object at 0x02A49ED0
            
            message2print = "{(Step 12/12b) inherited" + "; object-inherited-by-heap2xi-of-Heap4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #class 'object'  
            
            message2print = "{(Step 12/12c) shared" + "; alignment-shared-by-heap2xi-of-Heap4Xi: " + str(Heap4Xi._alignment)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)                      
         
            message2print = "{(Step 12/12d) returning" + "; init-of-Heap4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
        
                       
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "roundup" usage = "
    #   heap4xi.py
    #       _malloc(self, size) 
    #           length = self._roundup(max(self._size, size), mmap.PAGESIZE)">
    # ---------------------------------------------------------------------------------------------
    @staticmethod
    def _roundup(n, alignment):
        '''
        Heap4Xi method _roundup rounds queue length
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------    
        class2print = "Heap4Xi"
        method2print = "_roundup"
        
        message2print = "{started" + "; roundup-of-Heap4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed" + "; number-passed-to-roundup-of-Heap4Xi: " + str(n) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #4096...
        
        message2print = "{passed" + "; alignment-passed-to-roundup-of-Heap4Xi: " + str(alignment) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        #8...
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-roundup-performing-by-roundup-of-Heap4Xi: " + str(Heap._roundup) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            return Heap._roundup(n, alignment)
            

        else:             
        
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------          
            # alignment must be a power of 2
            mask = alignment - 1
    
            message2print = "{set" + "; mask-set-by-roundup-of-Heap4Xi to: " + str(mask)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #7...
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------  
            message2print = "{returning" + "; mask-returning-by-roundup-of-Heap4Xi: " + str((n + mask) & ~mask) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
            #4096...
            
            
            return (n + mask) & ~mask
    #</methodize4xi>
    
       
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "malloc" usage = "
    #   malloc <-- note not protected method such as the following
    #       (arena2xi, start2xi, stop2xi) = self._malloc(size)
    #">
    # ---------------------------------------------------------------------------------------------
    def _malloc(self, size):
        '''
        Heap4Xi method _malloc returns memory blocks
        '''
        # -----------------------------------------------------------------------------------------
        #(Step 1/21) OK
        # -----------------------------------------------------------------------------------------    
        class2print = "Heap4Xi"
        method2print = "_malloc"
        
        message2print = "{(Step 1/21a) started" + "; malloc-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 18:40:44 02/22/13 Pacific Standard Time
                       
        message2print = "{(Step 1/21b) passed" + "; size-passed-to-malloc-of-Heap4Xi: " + str(size) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # 4096
        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-malloc-performing-by-malloc-of-Heap4Xi: " + str(Heap._malloc) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            return Heap._malloc(self, size)
            
        else:            
            # -----------------------------------------------------------------------------------------
            #(Step 2/21) OK
            # -----------------------------------------------------------------------------------------          
            message2print = "{(Step 2/21) informing" + "; heap-block-informing-by-malloc-of-Heap4Xi: " + "heap-block returned might be much larger" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #heap-block returned might be much larger
            
            index2xi = bisect.bisect_left(self._lengths, size)
              
            # -----------------------------------------------------------------------------------------
            #(Step 3/21) OK
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 3/21) set" + "; index2xi-set-by-malloc-of-Heap4Xi to: " + str(index2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #0
            
            # -----------------------------------------------------------------------------------------
            #(Step 4/21) OK
            # -----------------------------------------------------------------------------------------           
            len2xi = len(self._lengths)
    
            message2print = "{(Step 4/21) set" + "; len2xi-set-by-malloc-of-Heap4Xi to: " + str(len2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #0     
            
            if index2xi == len2xi:
                
                # -----------------------------------------------------------------------------------------
                # (Step 5/21) OK
                # -----------------------------------------------------------------------------------------            
    
                length2xi = self._roundup(max(self._size, size), mmap.PAGESIZE)
    
                message2print = "{(Step 5/21) set" + "; length2xi-set-by-malloc-of-Heap4Xi to: " + str(length2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #4096
                
                # -----------------------------------------------------------------------------------------
                # (Step 6/21) OK
                # -----------------------------------------------------------------------------------------               
                self._size *= 2
                
     
                message2print = "{(Step 6/21) set" + "; size-of-heap2xi-set-by-malloc-of-Heap4Xi to: " + str(self._size)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #8192...
                            
    
                # -----------------------------------------------------------------------------------------
                #(Step 7/21) OK
                # -----------------------------------------------------------------------------------------                
               
                message2print = "{(Step 7/21a) passing" + "; length2xi-passing-to-Arena4Xi-by-malloc-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                #4096...    
                    
                message2print = "{(Step 7/21b) calling" + "; Arena4Xi-calling-by-malloc-of-Heap4Xi: " + str(Arena4Xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                #class 'arena4xi.Arena4Xi' 
                
                arena2xi = Arena4Xi(length2xi)
    
                # -----------------------------------------------------------------------------------------
                #(Step 8/21) OK
                # -----------------------------------------------------------------------------------------                
                         
                message2print = "{(Step 8/21) got" + "; arena2xi-got-by-malloc-of-Heap4Xi: " + str(arena2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #arena4xi.Arena4Xi object at 0x02988090
                
                # -----------------------------------------------------------------------------------------
                #(Step 9/21) OK
                # -----------------------------------------------------------------------------------------          
                self._arenas.append(arena2xi) 
                
              
                message2print = "{(Step 9/21) appended" + "; arena2xi-appended-by-malloc-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                #arena4xi.Arena4Xi object at 0x02988090
                                
                # -----------------------------------------------------------------------------------------
                #(Step 9/21) OK
                # -----------------------------------------------------------------------------------------               
                message2print = "{(Step 10/21) returning" + "; arena2xi-length2xi-returning-by-malloc-of-Heap4Xi: " + str((arena2xi, 0, length2xi)) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                #(arena4xi.Arena4Xi object at 0x02988090, 0, 4096)           
                
                return (arena2xi, 0, length2xi)
            else:
            
                # -----------------------------------------------------------------------------------------
                #(Step 11/21)
                # ----------------------------------------------------------------------------------------- 
                         
                length2xi = self._lengths[index2xi]
     
                message2print = "{(Step 11/21) set" + "; length2xi-set-by-malloc-of-Heap4Xi to: " + str(length2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #4096                
                                            
                seq2xi = self._len_to_seq[length2xi]
            
                # -----------------------------------------------------------------------------------------
                #(Step 12/21)
                # ----------------------------------------------------------------------------------------- 
                      
                message2print = "{(Step 12/21) got" + "; seq2xi-got-by-malloc-of-Heap4Xi: " + str(seq2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
    
                block2xi = seq2xi.pop() 
            
                # -----------------------------------------------------------------------------------------
                #(Step 13/21)
                # ----------------------------------------------------------------------------------------- 
                      
                message2print = "{(Step 13/21) got" + "; block2xi-got-by-malloc-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
                if not seq2xi:
            
                    # -------------------------------------------------------------------------------------
                    #(Step 14/21)
                    # ------------------------------------------------------------------------------------- 
                    message2print = "{(Step 14/21a) deleting" + "; length2xi-deletingby-malloc-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    message2print = "{(Step 14/21b) deleting" + "; lengths-deleting-by-malloc-of-Heap4Xi: " + str(self._lengths[index2xi]) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                      
                    del self._len_to_seq[length2xi], self._lengths[index2xi]
                    
                    
                                                           
            # -----------------------------------------------------------------------------------------
            #(Step 15/21)
            # -----------------------------------------------------------------------------------------  
                           
            (arena2xi, start2xi, stop2xi) = block2xi
            
            message2print = "{(Step 15/21) got" + "; arena2xi-got-by-malloc-of-Heap4Xi: " + str(arena2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                                                           
            # -----------------------------------------------------------------------------------------
            #(Step 16/21)
            # -----------------------------------------------------------------------------------------  
                
            message2print = "{(Step 16/21) got" + "; start2xi-got-by-malloc-of-Heap4Xi: " + str(start2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
                                                           
            # -----------------------------------------------------------------------------------------
            #(Step 17/21)
            # -----------------------------------------------------------------------------------------  
               
            message2print = "{(Step 17/21) got" + "; stop2xi-got-by-malloc-of-Heap4Xi: " + str(stop2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                                           
            # -----------------------------------------------------------------------------------------
            #(Step 18/21)
            # -----------------------------------------------------------------------------------------  
                
            message2print = "{(Step 18/21) got" + "; block2xi-got-by-malloc-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                                           
            # -----------------------------------------------------------------------------------------
            #(Step 19/21)
            # -----------------------------------------------------------------------------------------  
                
            message2print = "{(Step 19a/21) passing" + "; arena2xi-passing-to-del-allocated-blocks-by-malloc-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
            
            message2print = "{(Step 19b/21) passing" + "; start2xi-passing-to-del-allocated-blocks-by-malloc-of-Heap4Xi: " + str(start2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                                     
            del self._allocated_blocks[(arena2xi, start2xi)]
            
            # -----------------------------------------------------------------------------------------
            #(Step 20/21)
            # -----------------------------------------------------------------------------------------            
            message2print = "{(Step 20/21a) passing" + "; arena2xi-passing-to-del-stop-to-block-by-malloc-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(Step 20/21b) passing" + "; stop2xi-passing-to-del-stop-to-block-by-malloc-of-Heap4Xi: " + str(stop2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            
            del self._stop_to_block[(arena2xi, stop2xi)]
            
            
            # -----------------------------------------------------------------------------------------
            #(Step 21/21)
            # -----------------------------------------------------------------------------------------            
            message2print = "{(Step 21/21) returning" + "; block2xi-returning-by-malloc-of-Heap4Xi: " + str(block2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                    
            return block2xi
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "free-of-Heap4Xi" usage = "
    #     free(self, block) function in heap.py
    #        self._free(block)">
    # ---------------------------------------------------------------------------------------------
    def _free(self, block):
        ''' 
        Heap4Xi method _free frees location and try to merge with neighbours
        '''
        # -----------------------------------------------------------------------------------------
        # Step 1/6: OK
        # -----------------------------------------------------------------------------------------    
        class2print = "Heap4Xi"
        method2print = "_free"
        
        message2print = "{started" + "; free-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 19:23:24 02/22/13 Pacific Standard Time
        
        message2print = "{(Step 1/6a) passed" + "; block-passed-to-free-of-Heap4Xi: " + str(block) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # (<arena4xi.Arena4Xi object at 0x02A1D050>, 0, 4096) 
        # (<arena4xi.Arena4Xi object at 0x00000000034F5E80>, 4096, 8192)
        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-free-performing-by-free-of-Heap4Xi: " + str(Heap._free) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                
                             
            Heap._free(self, block)
            
        else: 
        
                
            (arena2xi, start2xi, stop2xi) = (None, None, None) 
            try:
                (arena2xi, start2xi, stop2xi) = block
    
                message2print = "{(Step 1/6b) set" + "; arena-block-set-by-free-of-Heap4Xi to: " + str((arena2xi, start2xi, stop2xi))  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # (<arena4xi.Arena4Xi object at 0x02A1D050>, 0, 4096)
                # (<arena4xi.Arena4Xi object at 0x00000000034F5E80>, 4096, 8192)
            except:
                message2print = "{(Step 1/6c) hit" + "; exception-hit-by-free-of-Heap4Xi: " + " *** " + str((arena2xi, start2xi, stop2xi))  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
                            
            # -----------------------------------------------------------------------------------------
            # Step 2/6: TO BE SOLVED JATKA
            # -----------------------------------------------------------------------------------------          
            message2print = "{(Step 2/6a) do" + "; freeing-of-blocks-do-by-free-of-Heap4Xi: " + "** unresolved freeing blocks in heap4xi _free method **" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            if self._stop_to_block == {}:
                self._skip = True
    
                message2print = "{(Step 2/6b) set" + "; skip-of-heap2xi-set-by-free-of-Heap4Xi to: " + str(self._skip)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #True
                  
            else:    
                try:
             
                    message2print = "{(Step 2/6c) trying" + "; block-previous-setting-trying-by-free-of-Heap4Xi: " + "trying to set previous block"  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)         
                                    
                    # { Heap4Xi : _free ; Step 2/6b trying to set; prev_block2xi time: 12:26:30 06/27/09 FLE Daylight Time}
                    
       
                    message2print = "{(Step 2/6d) got" + "; arena2xi-got-by-free-of-Heap4Xi: " + str(arena2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                    
                    # { Heap4Xi : _free ; Step 2/6c arena2xi: <arena4xi.Arena4Xi object at 0x000000000367D780>; time: 12:36:15 06/27/09 FLE Daylight Time}
                    
        
                    message2print = "{(Step 2/6e) got" + "; start2xi-got-by-free-of-Heap4Xi: " + str(start2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                    
                    # { Heap4Xi : _free ; Step 2/6d start2xi: 16; time: 12:36:15 06/27/09 FLE Daylight Time}
                    
                    message2print = "{(Step 2/6f) got" + "; stop-to-block-got-by-free-of-Heap4Xi: " + str(self._stop_to_block)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                         
                    # { Heap4Xi : _free ; Step 2/6e self._stop_to_block: {}; time: 12:36:15 06/27/09 FLE Daylight Time} <-- to be solved; why nott 4096 ????? !!!!
                    
                                 
                    prev_block2xi = self._stop_to_block[(arena2xi, start2xi)]
                    
                    message2print = "{(Step 2/6g) set" + "; prev_block2xi-set-by-free-of-Heap4Xi to: " + str(prev_block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
          
                    # { Heap4Xi : _free ; Step 2/6b trying to set; prev_block2xi time: 12:01:38 06/27/09 FLE Daylight Time}
                                  
                except:
    
                    message2print = "{(Step 2/6h) hit" + "; exception-hit-by-free-of-Heap4Xi: " + " *** " + "failed to set prev_block2xi in _free method of Heap4Xi "  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
               
                    # { Heap4Xi : _free ; Step 2/6g exception ***  failed to set; prev_block2xi *** time: 12:36:15 06/27/09 FLE Daylight Time}
                    pass
                else:
                    message2print = "{(Step 2/6i) passing" + "; prev_block2xi-passing-to-absorb-by-free-of-Heap4Xi: " + str(prev_block2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    
                                    
                    start2xi, _ = self._absorb(prev_block2xi)
                    
                    message2print = "{(Step 2/6j set" + "; start2xi-set-by-free-of-Heap4Xi to: " + str(start2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                                    
            # -----------------------------------------------------------------------------------------
            # Step 3/6
            # -----------------------------------------------------------------------------------------              
            if self._skip is True:
            
                message2print = "{(Step 3/6) skipped" + "; step-3-of-6-skipped-by-free-of-Heap4Xi: " + "*** Step 3/6 in free-of-Heap4Xi skipped - to be solved ***"  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                pass
            else:
      
                message2print = "{(Step 3/6a) entering" + "; next-block-setting-entering-by-free-of-Heap4Xi: " + " entering to set next block " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                try:
                    next_block2xi = self._start_to_block[(arena2xi, stop2xi)]
                  
                    message2print = "{(Step 3/6b) set" + "; next_block2xi-set-by-free-of-Heap4Xi to: " + str(next_block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                                    
                except:
                    message2print = "{(Step 3/6c) yielded" + "; exception-yieldedt-by-free-of-Heap4Xi: " + " * yielded to set next_block2xi in _free method of Heap4Xi * " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
            
                    pass
                else:
    
                    message2print = "{(Step 3/6d) passing" + "; next_block2xi-passing-to-stop-block-by-free-of-Heap4Xi: " + str(next_block2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    
                                                   
                    _, stop2xi = self._absorb(next_block2xi)
                    
                    message2print = "{(Step 3/6e) set" + "; stop2xi-set-by-free-of-Heap4Xi to: " + str(stop2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                                                 
        
          
                message2print = "{(Step 3/6f) passing" + "; arena2xi-passing-to-set-block2xi-by-free-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 3/6g) passing" + "; start2xi-passing-to-set-block2xi-by-free-of-Heap4Xi: " + str(start2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 3/6h) passing" + "; stop2xi-passing-to-set-block2xi-by-free-of-Heap4Xi: " + str(stop2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                          
                                    
                block2xi = (arena2xi, start2xi, stop2xi)
                
                message2print = "{(Step 3/6i) set" + "; block2xi-set-by-free-of-Heap4Xi to: " + str(block2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                    
                                
                length2xi = stop2xi - start2xi
        
                   
                message2print = "{(Step 3/6j) set" + "; length2xi-set-by-free-of-Heap4Xi to: " + str(length2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                    
                            
            # -----------------------------------------------------------------------------------------
            # Step 4/6
            # -----------------------------------------------------------------------------------------          
            if self._skip is True:
                message2print = "{(Step 4/6) skipped" + "; step-4-of-6-skipped-by-free-of-Heap4Xi: " + "** Step 4/6  in _free method of Heap4Xi skipped **"  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                      
            else:
            
       
                message2print = "{(Step 4/6a1) passing" + "; length2xi-passing-to-len-to-seq-by-free-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 4/6a1) passing" + "; block2xi-passing-to-len-to-seq-by-free-of-Heap4Xi: " + str(block2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                                
                try:
                    self._len_to_seq[length2xi].append(block2xi)
                    
                    message2print = "{(Step 4/6b) appended" + "; block2xi-appended-by-free-of-Heap4Xi: " + str(block2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                                    
                      
                except:
    
                    message2print = "{(Step 4/6c1) hit" + "; exception-hit-by-free-of-Heap4Xi: " + " *** Error in setting self._len_to_seq in _free method of Heap4Xi *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    
                    message2print = "{(Step 4/6c2) passing" + "; length2xi-passing-to-len-to-seq-by-free-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    
                    message2print = "{(Step 4/6c3) passing" + "; block2xi-passing-to-len-to-seq-by-free-of-Heap4Xi: " + str(block2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                       
                                        
                    self._len_to_seq[length2xi] = [block2xi]
                        
       
                    message2print = "{(Step 4/6d1) set" + "; len-to-seq-set-by-free-of-Heap4Xi to: " + str(self._len_to_seq[length2xi])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    message2print = "{(Step 4/6d2) passing" + "; lengths-passing-to-bisect-insort-by-free-of-Heap4Xi: " + str(self._lengths) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    
                    message2print = "{(Step 4/6d3) passing" + "; length2xi-passing-to-bisect-insort-by-free-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                                           
                    bisect.insort(self._lengths, length2xi)
                    
                    message2print = "{(Step 4/6e) performed" + "; bisect-insort-performed-by-free-of-Heap4Xi: " + str(bisect.insort) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                                          
                              
                    message2print = "{(Step 4/6f) hit" + "; exception-hit-by-free-of-Heap4Xi: " + " *** failed in setting self._len_to_seq and to perform bisect.insort *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                        
                        
                                    
            # -----------------------------------------------------------------------------------------
            # Step 5/6
            # -----------------------------------------------------------------------------------------     
            if self._skip is True:
    
                message2print = "{(Step 5/6a) skipped" + "; step-5-of-6-skipped-by-free-of-Heap4Xi: " + "** skipped Step 5/6 in _free method of Heap4Xi***"  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                       
            else:
            
                message2print = "{(Step 5/6a) entering; trial-to-set-block: setting block in _free method of Heap4Xi; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 5/6a1) passing" + "; arena2xi-passing-to-allocated-blocks-by-free-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 5/6a2) passing" + "; start2xi-passing-to-allocated-blocks-by-free-of-Heap4Xi: " + str(start2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                
                message2print = "{(Step 5/6a3) passing" + "; block2xi-passing-to-allocated-blocks-by-free-of-Heap4Xi: " + str(block2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                             
                try:
                    self._start_to_block[(arena2xi, start2xi)] = block2xi
                      
                    message2print = "{(Step 5/6b) set" + "; allocated-blocks-set-by-free-of-Heap4Xi to: " + str(self._allocated_blocks[(arena2xi, start2xi)])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
    
                    self._stop_to_block[(arena2xi, stop2xi)] = block2xi
                    
                    message2print = "{(Step 5/6c) set" + "; self._stop_to_block[(arena2xi, stop2xi)] to: " + str(self._stop_to_block[(arena2xi, stop2xi)])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    
                    message2print = "{(Step 5/6c) set" + "; stop-to-block-set-by-free-of-Heap4Xi to: " + str(self._stop_to_block[(arena2xi, stop2xi)])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    
                except:
    
                    message2print = "{(Step 5/6d) hit" + "; exception-hit-by-free-of-Heap4Xi: " + " *** failed to set; self._allocated_blocks and self._stop_to_block *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                        
                            
            # -----------------------------------------------------------------------------------------    
            # Step 6/6
            # -----------------------------------------------------------------------------------------   
            if self._skip is True:
                message2print = "{(Step 6/6a) skipped" + "; step-6-of-6-skipped-by-free-of-Heap4Xi: " + "** skipped steps in _free method of Heap4Xi - to be solved **"  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # { Heap4Xi : _free ; Step 6/6 Skipped some steps - to be solved - returning time: 13:49:48 06/27/09 FLE Daylight Time} 
            else:
               message2print = "{(Step 6/6b) got" + "; skip-got-by-free-of-Heap4Xi to: " + str(self._skip)  + "; time: " + time4xi() + "}"
               print4xi(class2print, method2print, message2print)
               # 
                    
            message2print = "{(Step 6/6c) returning" + "; free-of-Heap4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
                
                                                   
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "absorb" usage = "
    #      _free(self, block) function in heap.py
    #             start, _ = self._absorb(prev_block2xi)">
    # ---------------------------------------------------------------------------------------------
    def _absorb(self, block):
        '''
        Heap4Xi method _absorb returns memory block to absorved with other free blocks
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/5) 
        # -----------------------------------------------------------------------------------------    
        class2print = "Heap4Xi"
        method2print = "_absorb"
         
        message2print = "{(Step 1/5a) started" + "; absorb-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/5b) passed" + "; block-passed-to-absorb-of-Heap4Xi: " + str(block) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-absorb-performing-by-absorb-of-Heap4Xi: " + str(Heap._absorb) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                             
            return Heap._absorb(self, block)
            
        else:               
            # deregister this block so it can be merged with a neighbour
            
            (arena2xi, start2xi, stop2xi) = block
            
            message2print = "{(Step 1/5c) set" + "; arena2xi-start2xi-stop2xi-set-by-absorb-of-Heap4Xi to: " + str((arena2xi, start2xi, stop2xi))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                    
                            
            # -----------------------------------------------------------------------------------------
            # (Step 2/5) 
            # -----------------------------------------------------------------------------------------          
            message2print = "{(Step 2/5a) passing" + "; arena2xi-passing-to-del-by-absorb-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
            message2print = "{(Step 2/5b) passing" + "; start2xi-passing-to-del-by-absorb-of-Heap4Xi: " + str(start2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                         
            
            del self._allocated_blocks[(arena2xi, start2xi)]
            
            message2print = "{(Step 2/5c) passing" + "; arena2xi-passing-to-del-by-absorb-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{(Step 2/5d) passing" + "; stop2xi-passing-to-del-by-absorb-of-Heap4Xi: " + str(stop2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                            
            del self._stop_to_block[(arena2xi, stop2xi)]
                      
           
            # -----------------------------------------------------------------------------------------
            # (Step 3/5) 
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 3/5a) passing" + "; stop2xi-passing-to-length2xi-calculation-by-absorb-of-Heap4Xi: " + str(stop2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
            message2print = "{(Step 3/5b) passing" + "; start2xi-passing-to-length2xi-calculation-by-absorb-of-Heap4Xi: " + str(start2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                 
            
            length2xi = stop2xi - start2xi
            
    
            message2print = "{(Step 3/5c) calculated" + "; length2xi-calculated-by-absorb-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            seq2xi = self._len_to_seq[length2xi]
    
            message2print = "{(Step 3/5d) set" + "; seq2xi-set-by-absorb-of-Heap4Xi to: " + str(seq2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                    
            message2print = "{(Step 3/5e) passing" + "; block-passing-to-remove-by-absorb-of-Heap4Xi: " + str(block) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
                                                    
            seq2xi.remove(block)
            
            message2print = "{(Step 3/5f) performed" + "; remove-performed-by-absorb-of-Heap4Xi: " + str(seq2xi.remove) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                    
            
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/5) 
            # -----------------------------------------------------------------------------------------          
            if not seq2xi:
                message2print = "{(Step 4/5a) passing" + "; length2xi-passing-to-del-by-absorb-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                    
                            
                del self._len_to_seq[length2xi]
                
                message2print = "{(Step 4/5b) passing" + "; length2xi-passing-to-lengths-remove-by-absorb-of-Heap4Xi: " + str(length2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                           
                self._lengths.remove(length2xi)
                
                
            # -----------------------------------------------------------------------------------------
            # (Step 5/5) 
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 5/5a) returning" + "; start2xi-returning-by-absorb-of-Heap4Xi: " + str(start2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                           
            message2print = "{(Step 5/5b) returning" + "; stop2xi-returning-by-absorb-of-Heap4Xi: " + str(stop2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)       
            
            return start2xi, stop2xi
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_free_pending_blocks" usage = "free all the blocks in the pending list
    # - called with the lock held.
    #">
    # ---------------------------------------------------------------------------------------------
    def _free_pending_blocks(self):
        '''
        frees all the blocks in the pending list
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)  
        # -----------------------------------------------------------------------------------------    
        class2print = "Heap4Xi"
        method2print = "_free_pending_blocks"
        
        message2print = "{(Step 1/3a) started" + "; free-pending-blocks-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-free-pending-blocks-performing-by-free-pending-blocks-of-Heap4Xi: " + str(Heap._free_pending_blocks) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                             
            Heap._free_pending_blocks(self)
            
        else:              
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/3)  
            # -----------------------------------------------------------------------------------------            
            while True:
                try:
                    block2xi = self._pending_free_blocks.pop()
                    message2print = "{(Step 2/3a) got" + "; block2xi-got-by-free-pending-blocks-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except IndexError:
                    break
                
                message2print = "{(Step 2/3b) using" + "; block2xi-using-by-free-pending-blocks-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                try:
                    self._allocated_blocks.remove(block2xi)
                    
                    message2print = "{(Step 2/3c) using" + "; block2xi-using-by-free-pending-blocks-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:
                    message2print = "{(Step 2/3z) yielded" + "; block2xi-yielded-by-free-pending-blocks-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    break
                try:
                    
                    self._free(block2xi)
                    
                    message2print = "{(Step 2/3d) used" + "; block2xi-used-by-free-pending-blocks-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:
                    message2print = "{(Step 2/3z2) yielded" + "; block2xi-yielded-by-free-pending-blocks-of-Heap4Xi: " + str(block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    break                
                
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)   _free_pending_blocks a block returned by malloc()
            # -----------------------------------------------------------------------------------------          
            message2print = "{(Step 3/3) returning" + "; free-pending-blocks-of-Heap4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 

    #</methodize4xi>    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "free" usage = "frees blocks
    # 
    # Since free() can be called asynchronously by the GC, it could happen
    # that it's called while self._lock is held: in that case,
    # self._lock.acquire() would deadlock (issue #12352). To avoid that, a
    # trylock is used instead, and if the lock can't be acquired
    # immediately, the block is added to a list of blocks to be freed
    # synchronously sometimes later from malloc() or free(), by calling
    # _free_pending_blocks() (appending and retrieving from a list is not
    # strictly thread-safe but under cPython it's atomic thanks to the GIL).    
    #">
    # ---------------------------------------------------------------------------------------------
    def free(self, block):
        '''
        Heap4Xi method free frees blocks
        '''
        # -----------------------------------------------------------------------------------------
        # (Step 1/3)  
        # -----------------------------------------------------------------------------------------    
        class2print = "Heap4Xi"
        method2print = "free"
        
        message2print = "{(Step 1/3a) started" + "; free-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/3b) passed" + "; block-passed-to-free-of-Heap4Xi: " + str(block) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                         
        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-free-performing-by-free-of-Heap4Xi: " + str(Heap.free) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                             
            Heap.free(self, block)
            
        else:                
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/3)  
            # -----------------------------------------------------------------------------------------            
            ospid2xi = os.getpid()
            message2print = "{(Step 2/3a) got" + "; ospid2xi-got-by-free-of-Heap4Xi: " + str(ospid2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
                    
            lastpid2xi = self._lastpid
            message2print = "{(Step 2/3b) got" + "; lastpid2xi-got-by-free-of-Heap4Xi: " + str(lastpid2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            
            assert ospid2xi == lastpid2xi
            
            message2print = "{(Step 2/3c) asserted" + "; ospid2xi-asserted-by-free-of-Heap4Xi: " + str(ospid2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            
            message2print = "{(Step 2/3d) asserted" + "; lastpid2xi-asserted-by-free-of-Heap4Xi: " + str(lastpid2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/3) 
            # -----------------------------------------------------------------------------------------          
            if not self._lock.acquire(False):
                # can't acquire the lock right now, add the block to the list of
                # pending blocks to free
                try:
                    self._pending_free_blocks.append(block)
                    
                    message2print = "{(Step 3/3a) performed" + "; append-performed-by-free-of-Heap4Xi: " + str(self._pending_free_blocks.append)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                except:
                    message2print = "{(Step 3/3z1) failed" + "; block-failed-by-free-of-Heap4Xi: " + str(block)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                
            else:
                # we hold the lock
                try:
                    self._free_pending_blocks()
                    message2print = "{(Step 3/3b) performed" + "; free-pending-blocks-performed-by-free-of-Heap4Xi: " + str(self._free_pending_blocks)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    self._allocated_blocks.remove(block)
                    message2print = "{(Step 3/3c) performed" + "; remove-allocated-blocks-performed-by-free-of-Heap4Xi: " + str(self._allocated_blocks.remove)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    self._free(block)
                    
                    message2print = "{(Step 3/3d) performed" + "; free-performed-by-free-of-Heap4Xi: " + str(self._free)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                     
                except:
                    message2print = "{(Step 3/3z2) failed" + "; block-failed-by-free-of-Heap4Xi: " + str(block)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                finally:
                    self._lock.release()
                    
                    message2print = "{(Step 3/3a) performed" + "; release-performed-by-free-of-Heap4Xi: " + str(self._lock.release)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    message2print = "{(Step 3/3a) returning" + "; free-of-Heap4Xi-returning: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print) 

    #</methodize4xi>
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "malloc" usage = "
    #   heap.py
    #       BufferWrapper(object)
    #           _heap = Heap()
    #           def __init__(self, size):
    #               assert 0 <= size < sys.maxsize <--- does NOT seem to prevent allocation near 8giga in a system with 8 giga RAM system of which OS takes 2 giga 
    #               block = BufferWrapper._heap.malloc(size) <--- classes used for objects">
    # ---------------------------------------------------------------------------------------------
    def malloc(self, size):
        '''
        Heap4Xi method malloc returns memory allocated
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------    
        class2print = "Heap4Xi"
        method2print = "malloc"
        
        message2print = "{(Step 1/18a1) started" + "; malloc-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/18a2) passed" + "; size-passed-to-malloc-of-Heap4Xi: " + str(size) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # { Heap4Xi : malloc ; Step 1/18 size: 12; time: 13:55:23 06/25/09 FLE Daylight Time} 
        
        if examine4xi(class2print, method2print):  
        
            # -------------------------------------------------------------------------------------    
            # StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) performing" + "; Heap-malloc-performing-by-malloc-of-Heap4Xi: " + str(Heap.malloc) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                             
            return Heap.malloc(self, size)
            
        else:      
       
            maxsize2xi = sys.maxsize
    
            message2print = "{(Step 2/18a) got" + "; maxsize2xi-got-by-malloc-of-Heap4Xi: " + str(maxsize2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
    
            assert 0 <= size < maxsize2xi
            
    
            message2print = "{(Step 2/18b1) asserted" + "; size-asserted-by-malloc-of-Heap4Xi: " + str(size) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            message2print = "{(Step 2/18b1) asserted" + "; maxsize2xi-asserted-by-malloc-of-Heap4Xi: " + str(maxsize2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                         
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------          
            pid2os = os.getpid()
      
            message2print = "{(Step 3/18) got" + "; pid2os-got-by-malloc-of-Heap4Xi: " + str(pid2os)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 2332
            
            pid2self = self._lastpid
              
            message2print = "{(Step 4/18) set" + "; pid2heap2xi-set-by-malloc-of-Heap4Xi to: " + str(pid2self)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 2332  
            
            message2print = "{(Step 5/18a) comparing" + "; pid2os-comparing-by-malloc-of-Heap4Xi: " + str(pid2os) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # 2332
            
            message2print = "{(Step 5/18b) comparing" + "; pid2heap2xi-comparing-by-malloc-of-Heap4Xi: " + str(pid2self) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 2332
            
            if pid2os != pid2self:
             
                message2print = "{(Step 6/18) calling" + "; init-calling-by-malloc-of-Heap4Xi: " + str(self.__init__) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                    
                            
                self.__init__()                     # reinitialize after fork
            
      
                message2print = "{(Step 7/18) performed" + "; init-performed-by-malloc-of-Heap4Xi: " + str(self.__init__) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
      
            message2print = "{(Step 8/18a) acquiring" + "; lock-acquiring-by-malloc-of-Heap4Xi: " + str(self._lock.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            # <built-in method acquire of _thread.lock object at 0x0000000002B28030>
                        
            self._lock.acquire()
            
            message2print = "{(Step 8/18b) acquired" + "; lock-acquired-of-heap2xi-by-malloc-of-Heap4Xi: " + str(self._lock.acquire) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
            # <built-in method acquire of _thread.lock object at 0x0000000002B28030>
            
            self._free_pending_blocks()
            
            message2print = "{(Step 8/18c) performed" + "; free-pending-blocks-performed-of-heap2xi-by-malloc-of-Heap4Xi: " + str(self._free_pending_blocks) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <bound method Heap4Xi._free_pending_blocks of <heap4xi.Heap4Xi object at 0x0000000002AF16D8>>
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------          
            try:
    
                # -----------------------------------------------------------------------------------------    
                # Step 9/18: OK
                # -----------------------------------------------------------------------------------------    
             
            
                size = self._roundup(max(size,1), self._alignment)
                
    
                message2print = "{(Step 9/18) set" + "; size-set-by-malloc-of-Heap4Xi to: " + str(size)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 4096
                
                # -----------------------------------------------------------------------------------------    
                # Step 10 and 11 /18: OK
                # -----------------------------------------------------------------------------------------    
       
                message2print = "{(Step 10/18a) passing" + "; size-passing-to-arena-by-malloc-of-Heap4Xi: " + str(size) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 4096 
                
                (arena2xi, start2xi, stop2xi) = self._malloc(size)
                
                message2print = "{(Step 10/18b1) set" + "; arena2xi-set-by-malloc-of-Heap4Xi to: " + str(arena2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # <arena4xi.Arena4Xi object at 0x00000000035ED080>
                   
                message2print = "{(Step 10/18b2) set" + "; start2xi-set-by-malloc-of-Heap4Xi to: " + str(start2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                             
                # 0
                
                message2print = "{(Step 10/18b3) set" + "; stop2xi-set-by-malloc-of-Heap4Xi to: " + str(stop2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 4096
                
                new_stop2xi = start2xi + size
                
      
                message2print = "{(Step 11/18) set" + "; new_stop2xi-set-by-malloc-of-Heap4Xi to: " + str(new_stop2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 4096
                # -----------------------------------------------------------------------------------------    
                # Step 12/18 to be solved - see heap4xi.py
                # -----------------------------------------------------------------------------------------     
                 
                if new_stop2xi < stop2xi:
                    try:
                        message2print = "{(Step 12/18) calling; function: " + "self._free" + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                        message2print = "{(Step 12/18a) passing" + "; arena2xi-passing-to-free-by-malloc-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
                        
                        message2print = "{(Step 12/18b) passing" + "; new-stop2xi-passing-to-free-by-malloc-of-Heap4Xi: " + str(new_stop2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
                        
                        message2print = "{(Step 12/18c) passing" + "; stop2xi-passing-to-free-by-malloc-of-Heap4Xi: " + str(stop2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
    
                
                        self._free((arena2xi, new_stop2xi, stop2xi))
                    
     
                        message2print = "{(Step 13/18) performed" + "; free-of-heap2xi-performed-by-malloc-of-Heap4Xi: " + str(self._free) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
    
                            
                    except:
                        message2print = "{(Step 13/18) hit" + "; exception-hit-by-malloc-of-Heap4Xi: " + " *** failed in calling self._free *** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
    
                                        
                # -----------------------------------------------------------------------------------------    
                # Step 14/18 
                # -----------------------------------------------------------------------------------------    
                try: 
                    message2print = "{(Step 14/18a) passing" + "; arena2xi-passing-to-block2xi-by-malloc-of-Heap4Xi: " + str(arena2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    # arena2xi-passing-to-block2xi-by-malloc-of-Heap4Xi
                    
                    message2print = "{(Step 14/18b) passing" + "; start2xi-passing-to-block2xi-by-malloc-of-Heap4Xi: " + str(start2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                  
                    # 0
                    
                    message2print = "{(Step 14/18c) passing" + "; new-stop2xi-passing-to-block2xi-by-malloc-of-Heap4Xi: " + str(new_stop2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)              
                    # 4096
                    
                    block2xi = (arena2xi, start2xi, new_stop2xi)
                    
                    message2print = "{(Step 14/18) set" + "; block2xi-set-by-malloc-of-Heap4Xi to: " + str(block2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    # (<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096)
                                    
                    # -----------------------------------------------------------------------------------------    
                    #
                    # -----------------------------------------------------------------------------------------                             
                    message2print = "{(Step 15/18) passing" + "; block2xi-passing-to-add-block-by-malloc-of-Heap4Xi: " + str(block2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    # (<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096)
                    
                    self._allocated_blocks.add(block2xi)
                
     
                    message2print = "{(Step 16/18) performed" + "; allocated-blocks-add-of-heap2xi-performed-by-malloc-of-Heap4Xi: " + str(self._allocated_blocks.add) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    # <built-in method add of set object at 0x000000000356CD68>
                         
                except:
           
                    message2print = "{(Step 16/18) hit" + "; exception-hit-by-malloc-of-Heap4Xi: " + " *** failed in calling self._allocated_blocks.add in malloc of Heap4Xi *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                        
                                
                # -------------------------------------------------------------------------------------
                # 
                # -------------------------------------------------------------------------------------
                message2print = "{(Step 17/18) returning" + "; block2xi-returning-by-malloc-of-Heap4Xi: " + str(block2xi) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                # (<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096)
                
                return block2xi
                
            finally:
                self._lock.release()
    
                message2print = "{(Step 18/18) performed" + "; lock-release-performed-by-malloc-of-Heap4Xi: " + str(self._lock.release) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # <built-in method release of _thread.lock object at 0x0000000002B28030>                
                
                message2print = "{returning" + "; malloc-of-Heap4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                # 18:40:44 02/22/13 Pacific Standard Time            
                    
                                           
    #</methodize4xi>
                                           
    # ---------------------------------------------------------------------------------------------
    # sense
    #<methodize4xi method = "sense-of-Heap4Xi" usage = "senses Heap4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses Heap4Xi class objects
        '''
        
        
        class2print = 'Heap4Xi'
        method2print = 'sense'
        
        message2print = "{started" + "; sense-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; Heap4Xi-sensing-by-sense-of-Heap4Xi: " + str(Heap4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                       
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-Heap4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['lastpid-of-heap2xi'] = self._lastpid 
            kb2xi['lock-of-heap2xi'] = self._lock  
            kb2xi['size-of-heap2xi'] = self._size          
            kb2xi['lengths-of-heap2xi'] = self._lengths        
            kb2xi['len-to-seq-of-heap2xi'] = self._len_to_seq 
            kb2xi['allocated-blocks-of-heap2xi'] = self._allocated_blocks         
            kb2xi['start-to-block-of-heap2xi'] = self._start_to_block             
            kb2xi['stop-to-block-of-heap2xi'] = self._stop_to_block  
            kb2xi['arenas-of-heap2xi'] = self._arenas         
            
            kb2xi['inheritance-of-heap2xi'] = object 
            kb2xi['alignment-of-heap2xi'] = Heap4Xi._alignment
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-Heap4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
    
            
        message2print = "{sensed" + "; Heap4Xi-sensed-by-sense-of-Heap4Xi: " + str(Heap4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-Heap4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #{'lock-of-heap2xi': _thread.lock object at 0x02996EE0, 
        #'arenas-of-heap2xi': [arena4xi.Arena4Xi object at 0x02988090, arena4xi.Arena4Xi object at 0x02A549F0, arena4xi.Arena4Xi object at 0x02A59670, arena4xi.Arena4Xi object at 0x02A5E230], 
        #'stop-to-block-of-heap2xi': {}, 
        #'lengths-of-heap2xi': [],
        #'allocated-blocks-of-heap2xi': set(),
        #'object': 'heap4xi.Heap4Xi object at 0x02A49ED0',
        #'all-methods2xi': "['__class__', '__delattr__', '__eq__', '__format__', '__ge__', '__getattribute__', '__gt__', '__hash__', '__init__', '__le__', '__lt__', '__ne__', '__new__', '__reduce__', '__reduce_ex__', '__repr__', '__setattr__', '__sizeof__', '__str__', '__subclasshook__', '_absorb', '_free', '_malloc', '_roundup', 'ace', 'free', 'malloc', 'reason', 'sense']", 
        #'size-of-heap2xi': 65536,
        #'lastpid-of-heap2xi': 3020,
        #'inheritance-of-heap2xi': 'object-inheriting-by-heap2xi-of-Heap4Xi',
        #'start-to-block-of-heap2xi': {}, 
        #'len-to-seq-of-heap2xi': {}
        #}
                               
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    # reason
    #<methodize4xi method = "reason-of-Heap4Xi" usage = "reasons Heap4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons Heap4Xi class objects
        '''
         
        
        class2print = 'Heap4Xi'
        method2print = 'reason'

        message2print = "{started" + "; reason-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                             
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Heap4Xi-reasoning-by-reason-of-Heap4Xi: " + str(Heap4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-Heap4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['inheritance-level-of-heap2xi'] = "second-level-of-inheritance-by-heap2xi-of-Heap4Xi" 
        except:
            message2print = "{yielded" + "; kb4xi-yielded-by-reason-of-Heap4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
          
        
        message2print = "{reasoned" + "; Heap4Xi-reasoned-by-reason-of-Heap4Xi: " + str(Heap4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-Heap4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #{'inheritance-level-of-heap2xi': 'second-level-of-inheritance-by-heap2xi-of-Heap4Xi', 
        #'object': 'heap4xi.Heap4Xi object at 0x02A49ED0', 
        #'private-methods4xi': "['__class__', '__delattr__', '__eq__', '__format__', '__ge__', '__getattribute__', '__gt__', '__hash__', '__init__', '__le__', '__lt__', '__ne__', '__new__', '__reduce__', '__reduce_ex__', '__repr__', '__setattr__', '__sizeof__', '__str__', '__subclasshook__', '_absorb', '_free', '_malloc', '_roundup']"}
        
                                   
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    # ace
    #<methodize4xi method = "ace-of-Heap4Xi" usage = "aces Heap4Xi class objects">
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces Heap4Xi class objects 
        '''
               
        
        
        class2print = 'Heap4Xi'
        method2print = 'ace'
        
        message2print = "{started" + "; ace-of-Heap4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                                 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Heap4Xi-acing-by-ace-of-Heap4Xi: " + str(Heap4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                    
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-Heap4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------              
        try:
            kb8xi['inheritance-singularity-of-heap2xi'] = "single-inheritance-by-heap2xi-of-Heap4Xi"        
        except:
            message2print = "{yielded" + "; kb8xi-yielded-by-ace-of-Heap4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
          
        message2print = "{aced" + "; Heap4Xi-aced-by-ace-of-Heap4Xi: " + str(Heap4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-Heap4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #'public-methods8xi': "['ace', 'free', 'malloc', 'reason', 'sense']", 
        #'inheritance-singularity-of-heap2xi': 'single-inheritance-by-heap2xi-of-Heap4Xi', 
        # 'object8xi': 'heap4xi.Heap4Xi object at 0x02A49ED0'}                            
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_constant_heap4xi.txt
#   sense-test-constant-heap4xi.xml
#   reason-test-constant-heap4xi.xml
#   ace-test-constant-heap4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of heap4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

 